/**********************************************************************
$Id: botsfilter.c,v 1.4 2002/10/14 18:45:23 bu Exp $

Copyright 2001-2002, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "botsfilter.h"

#if DO_BOT_CHECKS

/* Variables are out here so that I can free them later */

static regex_t emailplatinum,twopointoxbot,newsgroupbulkmailer,calvacade98;
static regex_t atomicpost,botmessageidpattern,orgmidbotpattern,msgidabcxyzbot;
static regex_t nonepoundyetbot,strippathbot,adultsightsbot_org;
static regex_t adultsightsbot_msgid,repostbot;
static int botsfilter_compiled=0;

int compilebotregex() {

#if TESTOUTPUT
    cflog("botsfilter: About to see if regex_ts are compiled");
#endif

    if(!botsfilter_compiled) {
        if(!regcomp(&emailplatinum, "email\\s+platinum",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {
#if TESTOUTPUT
            cflog("botsfilter: emailplatinum Regexp loaded");
            
#endif
        } else return 0;

#if TESTOUTPUT
        cflog("botsfilter: about to load twopointoxbot");
        
#endif
                        
        if(!regcomp(&twopointoxbot, "^2\\.\\d\\.(\\d\\d? [A-Z]|\\d\\d?)$",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: twopointobot Regexp loaded");
            
#endif
        } else return 0;

#if TESTOUTPUT
        cflog("botsfilter: about to load newsgroupbulkmailer");
        
#endif        

        if(!regcomp(&newsgroupbulkmailer, "newsgroup\\s+bulk\\s+mailer",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: newsgroupbulkmailer Regexp loaded");
            
#endif
        } else return 0;
        
        if(!regcomp(&calvacade98, "calvacade 98",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: calvacade98 Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&atomicpost, "atomicpost",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: atomicpost Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&botmessageidpattern, "<\\d[12]\\@[A-Z]{10}>|\\@\\d+>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: botmessageidpattern Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&orgmidbotpattern, "<no\\s+organization>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: orgmidbotpattern Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&msgidabcxyzbot, "msgidabcxyz\\.com",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: msgidabcxyzbot Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&nonepoundyetbot, "none\\d+\\.yet>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: nonepoundyetbot Regexp loaded");
            
#endif
        } else return 0;        
        
        if(!regcomp(&strippathbot, "strip_path>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: strippathbot Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&adultsightsbot_org, "^[A-Z]{2,3}\\sInc\\.\\s*$",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: adultsightsbot_org Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&adultsightsbot_msgid, "<\\d{12}\\.\\d{10}\\@",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: adultsightsbot_msgid Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&repostbot, "repost.*unauthorized.*cancel",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: repostbot Regexp loaded");
            
#endif
        } else return 0;
        
#if TESTOUTPUT
        cflog("botsfilter: All patterns loaded");
                
#endif
        botsfilter_compiled=1;        
    }

    return 1;

}


int botsfilter(MSGPARTS *mymsg) {

    if(!botsfilter_compiled) {
    
        /* If regex compiles fail, return that it passed. */
        if(!compilebotregex()) return 0;
        
    }

#if TESTOUTPUT
    cflog("botsfilter: About to test patterns");
#endif

    if(mymsg->organization) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for organization");
        
#endif
        
        if(!regexec(&emailplatinum,mymsg->organization,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&repostbot,mymsg->organization,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->x_newsreader) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for x_newsreader");
        
#endif

        if(!regexec(&twopointoxbot,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&newsgroupbulkmailer,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&calvacade98,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->x_newsposter) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for x_newsposter");
        
#endif
        if(!regexec(&atomicpost,mymsg->x_newsposter,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->message_id) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for message_id");
        
#endif

        if(!regexec(&botmessageidpattern,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&nonepoundyetbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&strippathbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&nonepoundyetbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
    }
    if((mymsg->organization)&&(mymsg->message_id)) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for AdultSights");
        
#endif

        if((!regexec(&adultsightsbot_org,mymsg->organization,0,NULL,REG_NEWLINE))&&
           (!regexec(&adultsightsbot_msgid,mymsg->message_id,0,NULL,REG_NEWLINE))) return 1;
    }
    return 0;
}

void freebotregex() {

    if(botsfilter_compiled) {
        regfree(&emailplatinum);
        regfree(&twopointoxbot);
        regfree(&newsgroupbulkmailer);
        regfree(&calvacade98);
        regfree(&atomicpost);
        regfree(&botmessageidpattern);
        regfree(&orgmidbotpattern);
        regfree(&msgidabcxyzbot);
        regfree(&nonepoundyetbot);
        regfree(&strippathbot);
        regfree(&adultsightsbot_org);
        regfree(&adultsightsbot_msgid);
        regfree(&repostbot);
        
        botsfilter_compiled=0;
    }
}

#endif /* DO_BOT_CHECKS */