/**********************************************************************
$Id: userregex.c,v 1.2 2002/10/14 18:24:27 bu Exp $

Copyright 2002, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "userregex.h"

#if USE_USER_REGEX

struct userregex_t {
    char *regexname;
    regex_t tregex;
};

static struct userregex_t *userregs=NULL;
int regexcount=-1;

int loaduserregex() {

    FILE *regfile, *logfile;
    char linebuf[4096];
    char *namebuf, *regbuf;
    char *filename=NULL;
    
    regex_t tempregex;
    
    int lineno=0;
    
    filename=getuserregexfilename();
    
    if(filename!=NULL) {
        regfile=fopen(filename, "r");
    } else {
        free(filename);
    
        return 0;
    }
    
    if(regfile != NULL) {
        /* Read until the file is exhausted. */

        regexcount=0;
        
        while(!feof(regfile)) {
        
            fgets(&linebuf[0], 4095, regfile);
            
            lineno++;
            
            /* Check to see if line is empty or if it is a 'comment'. */
            
            if( (strlen(linebuf)>0) && (linebuf[0]!='#') ) {
                /* First look for the tab character to get the name. */
                
                namebuf=strtok(&linebuf[0], "\t");
                
                if(namebuf!=NULL) {
                    /* Now get the regular expression. */
                    
                    regbuf=strtok(NULL, "\t\r\n\0");
                    
                    if(regbuf!=NULL) {
                        /* Now I have my regular expression.  See if it compiles. */
                        
                        if(!regcomp(&tempregex, regbuf, REG_EXTENDED|REG_NOSUB|REG_NEWLINE)) {
                            /* First free the regular expression I just compiled. */
                            
                            regfree(&tempregex);
                            
                            /* The regular expression is good.  Now add it to my list. */
                            
                            regexcount++;
                            
                            userregs=realloc(userregs, regexcount*sizeof(struct userregex_t));
                            
                            if(userregs!=NULL) {
                                /* Insert some data into my struct. */
                                
                                userregs[regexcount - 1].regexname=strdup(namebuf);
                                regcomp(&userregs[regexcount - 1].tregex, regbuf, REG_EXTENDED|REG_NOSUB|REG_NEWLINE);
                            }
                        } else {
                            logfile=getloghandle();
                            if(logfile != NULL) fprintf(logfile, "Invalid regular expression in '%s' on line %d.\n", filename, lineno);
                        }
                    } else {
                        logfile=getloghandle();
                        if(logfile != NULL) fprintf(logfile, "Unrecognized format in '%s' on line %d.\n", filename, lineno);
                    }
                } else {
                    logfile=getloghandle();
                    if(logfile != NULL) fprintf(logfile, "Unrecognized format in '%s' on line %d.\n", filename, lineno);
                }
            }
        } /* while() */
    } else {
        cflog("Could not open regular expressions file.");
        
        return 0;
    }
    
    free(filename);
    
    return regexcount;

}


int userregex(char *article) {

    int i;

    if(regexcount==-1) loaduserregex();
    
    if(regexcount>0) {
        for(i=0; i<regexcount; i++) {
            if(!regexec(&userregs[i].tregex, article, 0, NULL, REG_NEWLINE)) {
#if TESTOUTPUT
                fprintf(getloghandle(), "Article caught by regex #%d\n", i+1);
#endif                
                return (i+10000);
            }
        }
    } 
#if TESTOUTPUT
    else cflog("No regular expressions to compare against.");
#endif    
    
    return 0;

}

void freeuserregex() {

    /* To be implemented. */

}

char *regexvalname(int filter) {
    
    if(filter<regexcount) return userregs[filter].regexname;
    else return NULL;

}

#endif /* USE_USER_REGEX */