/**********************************************************************
$Id: usieve.c,v 1.10 2002/10/14 18:24:27 bu Exp $

Copyright 2001-2002, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include <errno.h>
#include "usieve.h"

int usieve(char *article, long int artsize) {

    MSGPARTS currmsg;
    int reject=0;
#if TESTOUTPUT
    FILE *temphandle;
#endif

#if TESTOUTPUT
    cflog("Just read article");
    temphandle=getloghandle();
    fprintf(temphandle, "Article Length: %ld:\n",artsize);
    fflush(temphandle);
#endif

#if LOGLASTARTICLE
    logarticle(article,artsize);
#endif		        

    /* Split the article into its component parts. */

    decompose(article,&currmsg,artsize);

#if TESTOUTPUT
    fprintf(temphandle, "Out of decompose.\n");
    fflush(temphandle);
    if(currmsg.approved) fprintf(temphandle, "1: %s\n",currmsg.approved);
    if(currmsg.content_base) fprintf(temphandle, "2: %s\n",currmsg.content_base);
    if(currmsg.content_disposition) fprintf(temphandle, "3: %s\n",currmsg.content_disposition);
    if(currmsg.content_type) fprintf(temphandle, "4: %s\n",currmsg.content_type);
    if(currmsg.control) fprintf(temphandle, "5: %s\n",currmsg.control);
    if(currmsg.date) fprintf(temphandle, "6: %s\n",currmsg.date);
    if(currmsg.distribution) fprintf(temphandle, "7: %s\n",currmsg.distribution);
    if(currmsg.followup_to) fprintf(temphandle, "8: %s\n",currmsg.followup_to);
    if(currmsg.from) fprintf(temphandle, "9: %s\n",currmsg.from);
    if(currmsg.lines) fprintf(temphandle, "10: %d\n",currmsg.lines);
    if(currmsg.message_id) fprintf(temphandle, "11: %s\n",currmsg.message_id);
    if(currmsg.newsgroups) fprintf(temphandle, "12: %s\n",currmsg.newsgroups);
    if(currmsg.nntp_posting_host) fprintf(temphandle, "13: %s\n",currmsg.nntp_posting_host);
    if(currmsg.organization) fprintf(temphandle, "14: %s\n",currmsg.organization);
    if(currmsg.path) fprintf(temphandle, "15: %s\n",currmsg.path);
    if(currmsg.references) fprintf(temphandle, "16: %s\n",currmsg.references);
    if(currmsg.reply_to) fprintf(temphandle, "17: %s\n",currmsg.reply_to);
    if(currmsg.sender) fprintf(temphandle, "18: %s\n",currmsg.sender);
    if(currmsg.subject) fprintf(temphandle, "19: %s\n",currmsg.subject);
    if(currmsg.x_trace) fprintf(temphandle, "20: %s\n",currmsg.x_trace);
    if(currmsg.x_newsreader) fprintf(temphandle, "21: %s\n",currmsg.x_newsreader);
    if(currmsg.x_newsposter) fprintf(temphandle, "22: %s\n",currmsg.x_newsposter);
    if(currmsg.x_mailer) fprintf(temphandle, "23: %s\n",currmsg.x_mailer);
    if(currmsg.x_cancelled_by) fprintf(temphandle, "24: %s\n",currmsg.x_cancelled_by);
    fflush(temphandle);
#endif

    /* First test to see if we should automatically blackhole this article due
       to the path header.  Conditional compile for speed. */

#if BLACKHOLE_MATCH
    if(!reject) {
        if(currmsg.path) {
#if TESTOUTPUT
            temphandle=getloghandle();
            fprintf(temphandle, "blackhole: About to check path: %s", currmsg.path);
            fflush(temphandle);
#endif	 
            if(!blackholematch(currmsg.path)) {
#if TESTOUTPUT
                cflog("blackhole: Path did not match.  Performing other checks");
#endif
            } else {
#if TESTOUTPUT
                cflog("blackhole: Path Matched.  Auto-rejecting Message.");
#endif
                reject=2;
            }
        }
    }
#endif            

    /* Now check to see if I should blindly accept this article because I know
       that it was checked somewhere in the path it came from.  Conditional 
       compile for speed. */

#if PATH_MATCH
    if(!reject) {
        if(currmsg.path) {
#if TESTOUTPUT
            temphandle=getloghandle();
            fprintf(temphandle, "auto-accept: About to check path: %s", currmsg.path);
            fflush(temphandle);
#endif	 
            if(!pathmatch(currmsg.path)) {
#if TESTOUTPUT
                cflog("auto-accept: Path did not match.  Performing other checks");
#endif
            } else {
#if TESTOUTPUT
                cflog("auto-accept: Path Matched.  Auto-accepting Message.");
#endif
                reject=-1;
            }
        }
    }
#endif

    /* Now perform checks to make sure it isn't a binary article in a 
       non-binary group.  Conditional compile for speed.*/

#if BLOCK_BINARIES
    if(!reject) {
	    if(currmsg.newsgroups) {
            if(groupsbinarymatch(currmsg.newsgroups)) {
#if TESTOUTPUT
    	    cflog("Message is not in binary group... checking for binary in message");
#endif        	    
                if(ismessagebinary(&currmsg)) {
                    reject=3;
#if TESTOUTPUT                        
                    cflog("Message is binary in non-binary group. Rejecting...");
#endif                        
                }
            }
        }
    }
#endif

    /* Now make sure that the article doesn't exceed the admin specified number
       of cross-posts.  Conditional compile for speed. */    

#if MAX_CROSSPOSTS
    if(!reject) {
        if(maxcrosspost(currmsg.newsgroups)) {
#if TESTOUTPUT
            cflog("Message exceeded max crossposts.  Rejecting...");
#endif            
            reject=5;
        }
    }  
#endif /* MAX_CROSSPOSTS */

    /* Now check to make sure that the article wasn't posted by a spam bot. 
       Conditional compile for speed. */
    
#if DO_BOT_CHECKS
    if(!reject) {
        if(botsfilter(&currmsg)) {
#if TESTOUTPUT            
            cflog("Bot detected.  Rejecting...");
#endif                
            reject=4;           
        }
    }
#endif

    /* Now perform admin specified checks for regular expressions.  Conditional
       compile for speed. */

#if USE_USER_REGEX
    if(!reject) {
        if((reject=userregex(article))!=0) {
#if TESTOUTPUT            
            cflog("User Supplied Regular expression detected.  Rejecting...");
#endif                
	}
    }
#endif

    /* All checks have been run... Now check to make sure we haven't seen
       article before this.  Do this even for auto-accepted articles. 
       Conditional compile for speed. */

#if HISTORY_CHECK
    if(reject<=0) {
#if TESTOUTPUT
        cflog("Message cleared, about to run checksum logging...");
#endif

        if((summatch(currmsg.body,( artsize-(currmsg.body-article) )))==0) reject=6;
    

#if TESTOUTPUT
        cflog("Checksum cleared, accepting...");
#endif
    }
#endif /* HISTORY_CHECK */

    /* Free the memory used for the decomposed parts of the article. */

    freemsgparts(&currmsg);

    return reject;

}


int usieve_reload() {
    
    /* Perform free functions for each part of program for HUP. */
    
#if PATH_MATCH
    freepathmatchregex();
#endif
#if BLACKHOLE_MATCH
    freeblackholematchregex();
#endif

    return 0;
    
}

int usieve_term() {

    /* Perform free functions for each part of program for TERM. 
       This assumes that usieve_reload() gets run too. */
       
#if BLOCK_BINARIES
    freeblockbinariesregex();
#endif

#if USE_USER_REGEX
    freeuserregex();
#endif
    
#if HISTORY_CHECK
    dumptable();
#endif
    
    return 0;
}
